/*
*    Copyright (c) <2002-2003> <Jean-Philippe Barrette-LaPierre>
*    
*    Permission is hereby granted, free of charge, to any person obtaining
*    a copy of this software and associated documentation files 
*    (curlpp), to deal in the Software without restriction, 
*    including without limitation the rights to use, copy, modify, merge,
*    publish, distribute, sublicense, and/or sell copies of the Software,
*    and to permit persons to whom the Software is furnished to do so, 
*    subject to the following conditions:
*    
*    The above copyright notice and this permission notice shall be included
*    in all copies or substantial portions of the Software.
*    
*    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
*    OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
*    MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. 
*    IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY 
*    CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, 
*    TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
*    SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
*/

#ifndef REGITRY_BASE_HH
#define REGITRY_BASE_HH

#include "registry_manager.hpp"

namespace curlpp
{
   /**
    * This class is a registry for handles. you need to register curlpp 
    * handles to use the registry. When you register to this registry, all
    * changes to the registry is applied to the registred clients, for this
    * reason you cannot register on more than one registry, but
    * registry_manager check for that. See curlpp::registry_manager< T >
    * for more documentation.
    */
   template < typename T >
   class registry_base
   {
      public:
         friend class curlpp::registry_manager_impl< T >;
         
         /**
          * When deleted, a registry is unregistered from registry_manager,
          * all his handles are unregistered. See
          * curlpp::registry_manager< T > for more documentation.
          */
         virtual ~registry_base();
         
         /**
          * return the list of all registered handles on this
          * registry
          */
         std::list< T > handles();
         
         /**
          * This function register the handle to use this registry. It call
          * curlpp::registry_manager< T >::record( T *, registry_base< T > *). See
          * that function for more documentation.
          */
         void record
            ( const T &handle );
            
         /**
          * This function unregister the handle to the list of 
          * handle of this registry. It simply call
          * curlpp::registry_manager< T >::unrecord( T *, registry_base< T >
          * * ). See that function for more documentation.
          */
         void unrecord
            ( const T &handle );
            
      protected:
         /**
          * This function is called when something is changed in the 
          * registry, so we need to make the changes to all of the 
          * registered handles. You need to call this function when
          * a change is made on the curlpp::registry< T >
          */
         void refresh_all();
         
         /**
          * This is this function who really update a handle in the
          * registry. You need to override this function.
          */
         virtual void refresh( const T &handle );
         
      private:
         /**
          * This function add the handle to his list and it set the actual
          * option, if it's allready present it's simply do nothing. This
          * function is called by
          * curlpp::registry_manager_singleton< T >::record( T *, registry< T
          * > * )
          */
         void insert( const T &handle );
         
         /**
          * This function remove the handle to his list
          */
         void remove( const T &handle );
         
         
      private:
         std::list< T > m_handles;

      public:
         typedef ::utilspp::singleton_holder
            < 
               curlpp::registry_manager_impl< T >, 
               ::utilspp::creation_using_new,
               ::utilspp::lifetime_library,
               ::utilspp::threading_single
            > 
            registry_manager;
   };
   
};


#include "registry_manager_impl.inl"

#include "registry_manager.inl"

#include "registry_base.inl"


#endif //REGITRY_BASE_HH

