/*
*    Copyright (c) <2002-2003> <Jean-Philippe Barrette-LaPierre>
*    
*    Permission is hereby granted, free of charge, to any person obtaining
*    a copy of this software and associated documentation files 
*    (curlpp), to deal in the Software without restriction, 
*    including without limitation the rights to use, copy, modify, merge,
*    publish, distribute, sublicense, and/or sell copies of the Software,
*    and to permit persons to whom the Software is furnished to do so, 
*    subject to the following conditions:
*    
*    The above copyright notice and this permission notice shall be included
*    in all copies or substantial portions of the Software.
*    
*    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
*    OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
*    MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. 
*    IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY 
*    CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, 
*    TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
*    SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
*/

#ifndef REGISTRY_MANAGER_SINGLETON_HH
#define REGISTRY_MANAGER_SINGLETON_HH

#include <map>

namespace curlpp
{
   template < typename T >
   class registry_base;
   
   /**
    * This class keep a trace of where each request is registered ( in a
    * registry). An handle cannot be registered in more than one registry. 
    * We use a singleton here to be sure that there is not more than one
    * registry_manager.
    *
    * NOTE: Do NOT use directly this class, use curlpp::registry_manager< T >
    * instead.
    */
   template < typename T >
   class registry_manager_impl
   {
      public:
         /**
          * This function register the handle to use this registry.
          * If you try to register the same handle to an another
          * registry without unregistering it first, the function 
          * will before unregister it for you and after that it 
          * will register it to the new registry. It will silently
          * do nothing if you try to register twice the same 
          * handle to the same registry.
          */
         virtual void record
            (
               const T &handle,
               registry_base< T > *registry
            );
            
         /**
          * Use this function to get a list of registry where a handle 
          * is registered.
          */
         virtual std::list< registry_base< T > * > registries
            (
                  const T &handle
            );

         /**
          * This function unregister the handle from the registry.
          * If you unregister twice the same handle, it will 
          * unregister it only once, the second time it will 
          * simply do nothing. 
          *
          * NOTE 1: If your handle is deleted, it will automatically be
          * unregistered, so you're not obliged to unregister a 
          * handle before deleting it
          *
          * NOTE 2: If the registry on witch your registered is deleted
          * it will automatically unregister all of his handles.
          */
         virtual void unrecord
            ( const T &handle, registry_base< T > *registry );

         /**
          * This function unregister the handle from all the registries on
          * where he is registered.
          *
          * NOTE 1: If your handle is deleted, it will automatically be
          * unregistered, so you're not obliged to unregister a 
          * handle before deleting it.
          */
         virtual void unrecord( const T &handle );

      //private:
         /**
          * The destructor is private in order to be sure that nobody will
          * succeed to delete it by himself
          */
         virtual ~registry_manager_impl();

      private:
         std::multimap< T, registry_base< T > * > m_registry;
   };
};

#endif //REGISTRY_MANAGER_SINGLETON_HH

